/*
 * Copyright (c) 2001-2004 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: barrier_inter.c,v 1.1 2004/03/09 20:07:49 adharurk Exp $
 *
 * Program to test Intercommunicator Barrier
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <mpi.h> 
#include <lamtest_error.h>

int main(int argc, char **argv ) 
{ 
  int size, rank, port_length, color;
  MPI_Comm comm, intercomm;
  char *port_name;
  int buf = 1;

  MPI_Init(&argc, &argv);

  lamtest_need_even(__FILE__, __LINE__);
  lamtest_check_size(__FILE__, __LINE__, 2, 1);

  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  /* Even rank gets 1 while odd ranks get 0 value for color */
  color = (rank % 2) ? 0 : 1;
  MPI_Comm_split(MPI_COMM_WORLD, color, rank, &comm);
  
  if (color) {
    /* even ranks */
    port_name = malloc(MPI_MAX_PORT_NAME * sizeof(char));
    MPI_Open_port(MPI_INFO_NULL, port_name);
    port_length = strlen(port_name) + 1;

    MPI_Send(&port_length, 1, MPI_INT, rank + 1, 1, MPI_COMM_WORLD);
    MPI_Send(port_name, port_length, MPI_CHAR, rank + 1, 1, MPI_COMM_WORLD);

    MPI_Comm_accept(port_name, MPI_INFO_NULL, 0, comm, &intercomm);

    MPI_Barrier(intercomm);

    MPI_Comm_disconnect(&intercomm);
    MPI_Close_port(port_name);
    free(port_name);

  } else {
    /* odd ranks */
    MPI_Recv(&port_length, 1, MPI_INT, rank - 1, 1, MPI_COMM_WORLD, 
	     MPI_STATUS_IGNORE);

    /* allocate memory for the the port name */
    port_name = malloc(port_length * sizeof(char));
    MPI_Recv(port_name, port_length, MPI_CHAR, rank - 1, 1, MPI_COMM_WORLD, 
	     MPI_STATUS_IGNORE);

    /* connect to the server */
    MPI_Comm_connect(port_name, MPI_INFO_NULL, 0, comm, &intercomm);

    MPI_Barrier(intercomm);

    MPI_Comm_disconnect(&intercomm);
    free(port_name);
  }

  MPI_Comm_free(&comm);
  MPI_Finalize();
  return 0;
} 
